<?php

namespace DTInternet\DtiUsermanagement\Utility;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2022 Stefan Bublies <bublies@dt-internet.de>
 *
 *  All rights reserved
 *
 ***************************************************************/

use TYPO3\CMS\Core\Crypto\PasswordHashing\PasswordHashFactory;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\DebugUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Context\Context;
use TYPO3\CMS\Extbase\Domain\Repository\FrontendUserRepository;

class FeuserUtility
{

    /**
     * @param $registerData
     * @return array
     * (c) 2022 by Stefan Bublies@dt-internet.de
     */
    public static function validateAndStoreFeUserData($registerData)
    {
        // Store the FE-User in fe_users-table
        $connectionPool = GeneralUtility::makeInstance(ConnectionPool::class);
        /** @var \TYPO3\CMS\Core\Database\Query\QueryBuilder $queryBuilder */
        $queryBuilder = $connectionPool->getQueryBuilderForTable('fe_users');
        $queryBuilder
            ->getRestrictions()
            ->removeAll();
        $numberOfEntries = $queryBuilder
            ->count('uid')
            ->from('fe_users')
            ->where(
                $queryBuilder->expr()->like('username', '"%' . $registerData['username'] . '%"')
            )
            ->execute()
            ->fetchColumn();

        $time = time();
        $hashInstance = GeneralUtility::makeInstance(PasswordHashFactory::class)->getDefaultHashInstance('FE');
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('fe_users');
        if ($numberOfEntries > 0) {
            $queryBuilder
                ->getRestrictions()
                ->removeAll();
            $results = $queryBuilder
                ->select('*')
                ->from('fe_users')
                ->where(
                    $queryBuilder->expr()->like('username', '"%' . $registerData['username'] . '%"')
                )
                ->execute()
                ->fetchAll(\Doctrine\DBAL\FetchMode::ASSOCIATIVE);
            return $results;
        } else {
            if ($registerData['notanonymus'] == 1) {
                $usergroup = 3; // Muss noch per Konstante variabel gemacht werden
            } else {
                $usergroup = 2; // Muss noch per Konstante variabel gemacht werden
            }
            $addFeUser = $queryBuilder
                ->insert('fe_users')
                ->values([
                    'pid' => 16, // Muss noch per Konstante variabel gemacht werden
                    'first_name' => $registerData['firstname'],
                    'last_name' => $registerData['lastname'],
                    'email' => $registerData['email'],
                    'zip' => $registerData['zip'],
                    'username' => $registerData['username'],
                    'password' => $hashInstance->getHashedPassword($registerData['password']),
                    'tstamp' => $time,
                    'crdate' => $time,
                    'known' => empty($registerData['known']) ? ($registerData['known'] ?: 0): $registerData['known'],
                    'secretkey' => $registerData['secretkey'],
                    'usergroup' => $usergroup,
                ])
                ->execute();
            return 'add';
        }
    }

    /**
     * @param $registerData
     * @return true
     */
    public static function updateUser($registerData)
    {
        // Store the FE-User in fe_users-table
        $connectionPool = GeneralUtility::makeInstance(ConnectionPool::class);
        /** @var \TYPO3\CMS\Core\Database\Query\QueryBuilder $queryBuilder */
        $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('fe_users');
        $time = time();
        if (!empty($registerData['email'])) {
            $usergroup = 3; // Muss noch per Konstante variabel gemacht werden
        } else {
            $usergroup = 2; // Muss noch per Konstante variabel gemacht werden
        }
        $connection->update(
            'fe_users',
            [
                'tstamp' => $time,
                'first_name' => $registerData['firstname'],
                'last_name' => $registerData['lastname'],
                'email' => $registerData['email'],
                'zip' => $registerData['zip'],
                'known' => $registerData['known'],
                'secretkey' => $registerData['secretkey'],
                'usergroup' => $usergroup,
            ],
            [
                'uid' => $registerData['userid']
            ]
        );

        return true;
    }

    /**
     * @param $uid
     * @return true
     */
    public static function deleteUser($uid)
    {
        // Store the FE-User in fe_users-table
        $connectionPool = GeneralUtility::makeInstance(ConnectionPool::class);
        /** @var \TYPO3\CMS\Core\Database\Query\QueryBuilder $queryBuilder */
        $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('fe_users');
        $time = time();
        $connection->update(
            'fe_users',
            [
                'deletedate' => $time,
                'deleted' => 1
            ],
            [
                'uid' => $uid
            ]
        );

        return true;
    }

    /**
     * @return array
     */
    public static function get() {
        return self::getCurrentUser();
    }

    /**
     * Array mit den Daten des aktuellen FE-Users holen.
     * @return array
     */
    public static function getCurrentUser() {
        if (!self::isLoggedIn()) return [];

        // Wenn wir ein Frontend haben, sind die fe_user-Daten global und vollständig im TSFE gespeichert
        if ((($GLOBALS['TSFE'] ?? false) && $GLOBALS['TSFE']->fe_user)) {
            return $GLOBALS['TSFE']->fe_user->user ?? [];
        }

        // Ohne Frontend könnten wir uns z.B. in einer Middleware befinden. Nach AUTH sind die Daten evtl im Aspect.
        $context = GeneralUtility::makeInstance(Context::class);
        $userAspect = $context->getAspect('frontend.user');
        if (!$userAspect) return [];


        // Daten zu Standard-Darstellung normalisieren
        return [
            'uid'			=> $userAspect->get('id'),
            'username'		=> $userAspect->get('username')
        ] ?? [];
    }

    /**
     * Prüft, ob der User aktuell als FE-User eingeloggt ist.
     * Damals: isset($GLOBALS['TSFE']) && $GLOBALS['TSFE']->loginUser
     * @return boolean
     */
    public static function isLoggedIn() {

        // Context `frontend.user.isLoggedIn` scheint in Middleware nicht zu gehen. Fallback auf TSFE.
        $loginUserFromTsfe = (isset($GLOBALS['TSFE']) && isset($GLOBALS['TSFE']->fe_user) && isset($GLOBALS['TSFE']->fe_user->user['uid']));

        $context = GeneralUtility::makeInstance(Context::class);
        return $context->getPropertyFromAspect('frontend.user', 'isLoggedIn') || $loginUserFromTsfe;
    }

    /**
     * Aktuellen FE-User ausloggen
     * @return void
     */
    public function logout() {
        if (!$this->isLoggedIn()) return false;

        // In der MiddleWare ist der FE-User evtl. noch nicht initialisiert...
        if ($TSFE = \dti\typo3::Tsfe()->get()) {
            if ($TSFE->fe_user && $TSFE->fe_user->logoff) {
                $TSFE->fe_user->logoff();
            }
        }

        // Session-Daten aus Tabelle `fe_sessions` löschen
        if ($sessionManager = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Session\SessionManager::class)) {
            $sessionBackend = $sessionManager->getSessionBackend('FE');
            $sessionBackend->remove( $this->getSessionId() );
        }

        // ... aber Cookie löschen geht immer!
        $this->removeCookie();

        // ToDo: Replace with Signal/Slot when deprecated
        if ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['felogin']['logout_confirmed']) {
            $_params = array();
            foreach ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['felogin']['logout_confirmed'] as $_funcRef) {
                if ($_funcRef) GeneralUtility::callUserFunction($_funcRef, $_params, $this);
            }
        }
    }

    /**
     * Session-ID des aktuellen FE-Users
     * @return string
     */
    public function getSessionId(){
        if ($sessionId = $GLOBALS['TSFE']->fe_user ? $GLOBALS['TSFE']->fe_user->id : null) {
            return $sessionId;
        }
        return $_COOKIE[$this->getCookieName()] ?? null;
    }


    /**
     * Aktuellen `fe_typo_user`-Cookie manuell löschen
     * @return void
     */
    public function removeCookie() {
        $cookieDomain = \dti\typo3::Helper()->getCookieDomain();
        $cookiePath = $cookieDomain ? '/' : GeneralUtility::getIndpEnv('TYPO3_SITE_PATH');
        setcookie('fe_typo_user', null, -1, $cookiePath, $cookieDomain);
        unset($_COOKIE['fe_typo_user']);
    }

    /**
     * Cookie-Name des Frontend-User-Cookies auslesen.
     * Normalerweise `fe_typo_user`, außer er wurde geändert.`
     * return string
     */
    public function getCookieName() {
        if ($cookieName = $GLOBALS['TYPO3_CONF_VARS']['FE']['cookieName'] ?? false) {
            return $cookieName;
        }
        return \dti\typo3::Helper()->getLocalConf('FE.cookieName');
    }

    /**
     * Update state from agents
     * @return true
     */
    public static function updateState($feuserData, $formData)
    {
        // Store the FE-User in fe_users-table
        $connectionPool = GeneralUtility::makeInstance(ConnectionPool::class);
        /** @var \TYPO3\CMS\Core\Database\Query\QueryBuilder $queryBuilder */
        $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('fe_users');
        $connection->update(
            'fe_users',
            [
                'isonline' => empty($formData['isonline']) ? ($formData['isonline'] ?: 0): $formData['isonline'],
                'chatavailable' => empty($formData['chatavailable']) ? ($formData['chatavailable'] ?: 0): $formData['chatavailable']
            ],
            [
                'uid' => $feuserData['uid']
            ]
        );

        return true;
    }

    /**
     * @param $uids
     * @return mixed[]
     */
    public static function readFeUserData($uids)
    {
        $userRepository = GeneralUtility::makeInstance(FrontendUserRepository::class);
        $results = [];
        foreach($uids as $uid) {
            $results[] = $userRepository->findByUid($uid);
        }
        return $results;
    }

    /**
     * @param $uid
     * @return mixed[]
     */
    public static function readFeUserDataSingle($uid)
    {
        $userRepository = GeneralUtility::makeInstance(FrontendUserRepository::class);
        return $userRepository->findByUid($uid);
    }

    /**
     * @return mixed[]
     */
    public static function validateAgentsSession()
    {
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('fe_sessions');
        $queryBuilder
            ->getRestrictions()
            ->removeAll();
        $results = $queryBuilder
            ->select('ses_userid')
            ->from('fe_sessions')
            ->execute()
            ->fetchAll(\Doctrine\DBAL\FetchMode::ASSOCIATIVE);
        return $results;
    }
}
