<?php

namespace DTInternet\DtiUsermanagement\Utility;

use TYPO3\CMS\Core\TypoScript\TypoScriptService;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManager;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Core\Database\ConnectionPool;

/***************************************************************
 *  Copyright notice
 *  (c) 2022 Stefan Bublies <bublies@dt-internet.de>
 *  All rights reserved
 ***************************************************************/

class HelperUtility
{
    /**
     * 	Konfiguration aus der `LocalConfiguration.php` holen
     * 	@return string
     */
    public function getLocalConf ( $path = '' ) {
        if (!$path) return $GLOBALS['TYPO3_CONF_VARS'];
        return $this->getFromPath( $path, $GLOBALS['TYPO3_CONF_VARS'] ) ?: '';
    }

    /**
     * Setup von einem gegebenen Pfad holen, z.B. 'plugin.tx_example.settings'`
     * @return array
     */
    public function getFromPath( $tsPath = '', $setup = null ) {

        if (is_object($setup)) {
            $setup = (array) $setup;
        }

        $parts = \dti\typo3::Arrays($tsPath)->trimExplode('.');
        $setup = $setup ?: $this->getFullTyposcript();

        $root = array_shift($parts);
        $plugin = array_shift($parts);

        $setup = $setup[$root] ?? [];
        if (!$plugin) return $setup;

        $setup = $setup[$plugin] ?? [];
        if (!count($parts)) return $setup;

        while (count($parts) > 0) {
            $part = array_shift($parts);
            if (count($parts) == 0) {
                return isset($setup[$part]) && is_array($setup[$part]) ? $setup[$part] : ($setup[$part] ?? '');
            }
            $setup = $setup[$part];
        }

        return $setup;
    }

    /**
     * Das komplette TypoScript Setup holen, als einfaches Array - ohne "."-Syntax
     * Funktioniert sowohl im Frontend als auch Backend, mit und ohne übergebener pid`
     * @return array
     */
    public function getFullTyposcript( $pid = null ) {
        if ($this->typoscriptSetupCache) return $this->typoscriptSetupCache;

        $configurationManager = \dti\typo3::injectClass(ConfigurationManager::class);
        $setup = $configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT);

        // Setup konnte nicht geladen werden? Dann manuell erstellen
        if (!$setup) {
            $setup = $this->getTyposcriptObject( $pid )->setup ?: [];
        }

        return $this->typoscriptSetupCache = $this->convertToPlainArray($setup);
    }

    /**
     * TemplateService instanziieren, TypoScript-Config und Setup parsen.
     * Interne Funktion – nicht zur Verwendung gedacht.
     * `getFullTyposcript` nutzen.
     *
     * @return object
     */
    public function getTyposcriptObject ( $pid = null ) {

        if ($this->typoscriptObjectCache) return $this->typoscriptObjectCache;

        $rootline = \nn\t3::Page()->getRootline( $pid );

        $TsObj = GeneralUtility::makeInstance( \TYPO3\CMS\Core\TypoScript\ExtendedTemplateService::class );
        $TsObj->tt_track = 0;
        if (method_exists($TsObj, 'init')) $TsObj->init();
        $TsObj->runThroughTemplates($rootline);
        $TsObj->generateConfig();

        return $this->typoscriptObjectCache = $TsObj;
    }

    /**
     * 	TypoScript 'name.'-Syntax in normales Array umwandeln.
     * 	Erleichtert den Zugriff
     * 	@return array
     */
    public function convertToPlainArray ($ts) {
        if (!$ts || !is_array($ts)) return [];
        $typoscriptService = \dti\typo3::injectClass( TypoScriptService::class );
        return $typoscriptService->convertTypoScriptArrayToPlainArray($ts);
    }

    /**
     * 	Die Cookie-Domain holen z.B. www.dt-internet.de
     * 	@return string
     */
    public function getCookieDomain ( $loginType = 'FE' ) {
        $cookieDomain = $this->getLocalConf( $loginType . '.cookieDomain' )
            ?: $this->getLocalConf( 'SYS.cookieDomain' );
        return $cookieDomain;
    }

    /**
     * PID der aktuellen Seite holen.
     * Im Frontend: Die aktuelle `TSFE->id`
     * Im Backend: Die Seite, die im Seitenbaum ausgewählt wurde
     * Ohne Context: Die pid der site-Root
     * @return int
     */
    public function getPid ( $fallback = null ) {

        // Normaler Frontend-Content: Alles beim alten
        if (TYPO3_MODE == 'FE' && ($pid = $GLOBALS['TSFE']->id)) return $pid;

        // Versuch, PID über den Request zu bekommen
        if ($pid = $this->getPidFromRequest()) return $pid;

        // Context nicht klar, dann PID der Site-Root holen
        if ($siteRoot = $this->getSiteRoot()) return $siteRoot['uid'];

        // Letzte Chance: Fallback
        if ($fallback) return $fallback;
    }

    /**
     * PID der Site-Root(s) holen.
     * Entspricht der Seite im Backend, die die "Weltkugel" als Symbol hat
     * (in den Seiteneigenschaften "als Anfang der Webseite nutzen")
     * ```
     * @return int
     */
    public function getSiteRoot( $returnAll = false ) {
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('pages');
        $queryBuilder
            ->select('*')
            ->from('pages')
            ->andWhere($queryBuilder->expr()->eq('is_siteroot', '1'));

        if ($returnAll) return $queryBuilder->execute()->fetchAll();
        return $queryBuilder->execute()->fetch();
    }

    /**
     * PID aus Request-String holen, z.B. in Backend Modulen.
     * @return int
     */
    public function getPidFromRequest () {
        $pageUid = $GLOBALS['_REQUEST']['popViewId'] ?? false;
        if (!$pageUid) $pageUid = preg_replace( '/(.*)(id=)([0-9]*)(.*)/i', '\\3', $GLOBALS['_REQUEST']['returnUrl'] ?? '' );
        if (!$pageUid) $pageUid = preg_replace( '/(.*)(id=)([0-9]*)(.*)/i', '\\3', $GLOBALS['_POST']['returnUrl'] ?? '' );
        if (!$pageUid) $pageUid = preg_replace( '/(.*)(id=)([0-9]*)(.*)/i', '\\3', $GLOBALS['_GET']['returnUrl'] ?? '' );
        if (!$pageUid && ($_GET['edit']['pages'] ?? false)) $pageUid = array_keys($_GET['edit']['pages'] ?? [])[0];
        if (!$pageUid) $pageUid = $_GET['id'] ?? 0;
        return (int) $pageUid;
    }
}
